@@ -8,6 +8,7 @@ import sys
 from typing import Dict, List, Any
 import warnings
 
+import numpy as np
 import pandas as pd
 
 # opm and ecl2df are only available for Linux,
@@ -67,6 +68,9 @@ def filter_pvt_data_frame(
         "VOLUMEFACTOR_UNIT",
         "VISCOSITY",
         "VISCOSITY_UNIT",
+        "DENSITY",
+        "DENSITY_UNIT",
+        "RATIO_UNIT",
     ]
 
     if not "RATIO" in data_frame.columns:
@@ -74,11 +78,18 @@ def filter_pvt_data_frame(
             "The dataframe must contain a column for the ratio (OGR, GOR, R, RV, RS)."
         )
     if not "VOLUMEFACTOR_UNIT" in data_frame.columns:
-        data_frame["VOLUMEFACTOR_UNIT"] = "rm^3/sm^3"
+        data_frame["VOLUMEFACTOR_UNIT"] = "rm³/sm³"
     if not "PRESSURE_UNIT" in data_frame.columns:
         data_frame["PRESSURE_UNIT"] = "bar"
     if not "VISCOSITY_UNIT" in data_frame.columns:
         data_frame["VISCOSITY_UNIT"] = "cP"
+    if not "DENSITY_UNIT" in data_frame.columns:
+        data_frame["DENSITY_UNIT"] = "kg/m³"
+    if not "RATIO_UNIT" in data_frame.columns:
+        data_frame["RATIO_UNIT"] = "Scm³/Scm³"
+
+    if not "DENSITY" in data_frame.columns:
+        data_frame = calculate_densities(data_frame)
 
     data_frame = data_frame[columns]
 
@@ -132,6 +143,42 @@ def filter_pvt_data_frame(
     return cleaned_data_frame
 
 
+def calculate_densities(data_frame: pd.DataFrame) -> pd.DataFrame:
+    oil_density = data_frame.loc[
+        data_frame["KEYWORD"] == "DENSITY", "OILDENSITY"
+    ].values[0]
+    gas_density = data_frame.loc[
+        data_frame["KEYWORD"] == "DENSITY", "GASDENSITY"
+    ].values[0]
+    water_density = data_frame.loc[
+        data_frame["KEYWORD"] == "DENSITY", "WATERDENSITY"
+    ].values[0]
+
+    def calculate_density(keyword: str, ratio: float, volume_factor: float) -> float:
+        density = 0.0
+        if keyword == "PVTO":
+            density = (oil_density + ratio * gas_density) / volume_factor
+        elif keyword == "PVDO":
+            density = oil_density / volume_factor
+        elif keyword == "PVTG":
+            density = (gas_density + ratio * oil_density) / volume_factor
+        elif keyword == "PVDG":
+            density = gas_density / volume_factor
+        elif keyword == "PVCDO":
+            density = oil_density / volume_factor
+        elif keyword == "PVTW":
+            density = water_density / volume_factor
+        return density
+
+    data_frame["DENSITY"] = data_frame.apply(
+        lambda row: calculate_density(
+            row["KEYWORD"], row["RATIO"], row["VOLUMEFACTOR"]
+        ),
+        axis=1,
+    )
+    return data_frame
+
+
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
 @webvizstore
 def load_pvt_csv(
@@ -185,18 +232,21 @@ def load_pvt_dataframe(
         gas = Gas.from_ecl_init_file(ecl_init_file, True)
         water = Water.from_ecl_init_file(ecl_init_file, True)
 
-        column_pvtnum = []
-        column_ratio = []
-        column_volume_factor = []
-        column_volume_factor_unit = []
-        column_pressure = []
-        column_pressure_unit = []
-        column_viscosity = []
-        column_viscosity_unit = []
-        column_keyword = []
+        column_pvtnum: List[int] = []
+        column_ratio: List[float] = []
+        column_volume_factor: List[float] = []
+        column_volume_factor_unit: List[str] = []
+        column_pressure: List[float] = []
+        column_pressure_unit: List[str] = []
+        column_viscosity: List[float] = []
+        column_viscosity_unit: List[str] = []
+        column_density: List[float] = []
+        column_density_unit: List[str] = []
+        column_ratio_unit: List[str] = []
+        column_keyword: List[str] = []
 
-        ratios: List[float] = []
-        pressures: List[float] = []
+        ratios: np.ndarray = np.zeros(21)
+        pressures: np.ndarray = np.zeros(21)
         (pressure_min, pressure_max) = (0.0, 0.0)
 
         if oil and not oil.is_dead_oil_const_compr():
@@ -207,10 +257,10 @@ def load_pvt_dataframe(
             raise NotImplementedError("Missing PVT data")
 
         for pressure_step in range(0, 20 + 1):
-            pressures.append(
-                pressure_min + pressure_step / 20.0 * (pressure_max - pressure_min)
+            pressures[pressure_step] = pressure_min + pressure_step / 20.0 * (
+                pressure_max - pressure_min
             )
-            ratios.append(0.0)
+            ratios[pressure_step] = 0.0
 
         if oil:
             if oil.is_live_oil():
@@ -243,6 +293,9 @@ def load_pvt_dataframe(
                     column_viscosity_unit.extend(
                         [oil.viscosity_unit() for _ in pressures]
                     )
+                    column_density.extend(region.density(ratios, pressures))
+                    column_density_unit.extend([oil.density_unit() for _ in pressures])
+                    column_ratio_unit.extend([oil.ratio_unit() for _ in pressures])
 
                 else:
                     (ratio, pressure) = (
@@ -264,6 +317,9 @@ def load_pvt_dataframe(
                     column_viscosity_unit.extend(
                         [oil.viscosity_unit() for _ in pressure]
                     )
+                    column_density.extend(region.density(ratio, pressure))
+                    column_density_unit.extend([oil.density_unit() for _ in pressure])
+                    column_ratio_unit.extend([oil.ratio_unit() for _ in pressure])
 
         if gas:
             if gas.is_wet_gas():
@@ -300,6 +356,10 @@ def load_pvt_dataframe(
                 column_viscosity.extend(region.viscosity(ratio, pressure))
                 column_viscosity_unit.extend([gas.viscosity_unit() for _ in pressure])
 
+                column_density.extend(region.density(ratio, pressure))
+                column_density_unit.extend([gas.density_unit() for _ in pressure])
+                column_ratio_unit.extend([gas.ratio_unit() for _ in pressure])
+
         if water:
             for region_index, region in enumerate(water.regions()):
                 column_pvtnum.extend([region_index + 1 for _ in pressures])
@@ -317,6 +377,9 @@ def load_pvt_dataframe(
                 column_viscosity_unit.extend(
                     [water.viscosity_unit() for _ in pressures]
                 )
+                column_density.extend(region.density(ratio, pressures))
+                column_density_unit.extend([water.density_unit() for _ in pressures])
+                column_ratio_unit.extend(["" for _ in pressures])
 
         data_frame = pd.DataFrame(
             {
@@ -329,6 +392,9 @@ def load_pvt_dataframe(
                 "VOLUMEFACTOR_UNIT": column_volume_factor_unit,
                 "VISCOSITY": column_viscosity,
                 "VISCOSITY_UNIT": column_viscosity_unit,
+                "DENSITY": column_density,
+                "DENSITY_UNIT": column_density_unit,
+                "RATIO_UNIT": column_ratio_unit,
             }
         )
 
