@@ -1,6 +1,6 @@
+from typing import Optional, List, Dict, Any
 import sys
 import warnings
-from typing import Optional
 from pathlib import Path
 
 import pandas as pd
@@ -8,6 +8,8 @@ import numpy as np
 import yaml
 from webviz_config.utils import terminal_colors
 
+from .._utils.colors import hex_to_rgba
+
 
 def set_simulation_line_shape_fallback(line_shape_fallback: str) -> str:
     """
@@ -47,15 +49,17 @@ def get_simulation_line_shape(
     return line_shape_fallback
 
 
-def calc_series_statistics(df: pd.DataFrame, vectors: list, refaxis: str = "DATE"):
+def calc_series_statistics(
+    df: pd.DataFrame, vectors: list, refaxis: str = "DATE"
+) -> pd.DataFrame:
     """Calculate statistics for given vectors over the ensembles
     refaxis is used if another column than DATE should be used to groupby.
     """
     # Invert p10 and p90 due to oil industry convention.
-    def p10(x):
+    def p10(x: List[float]) -> List[float]:
         return np.nanpercentile(x, q=90)
 
-    def p90(x):
+    def p90(x: List[float]) -> List[float]:
         return np.nanpercentile(x, q=10)
 
     # Calculate statistics, ignoring NaNs.
@@ -86,10 +90,10 @@ def add_fanchart_traces(
     line_shape: str,
     refaxis: str = "DATE",
     hovertemplate: str = "(%{x}, %{y})<br>",
-):
+) -> List[Dict[str, Any]]:
     """Renders a fanchart for an ensemble vector"""
-    fill_color = hex_to_rgb(color, 0.3)
-    line_color = hex_to_rgb(color, 1)
+    fill_color = hex_to_rgba(color, 0.3)
+    line_color = hex_to_rgba(color, 1)
     return [
         {
             "name": legend_group,
@@ -152,16 +156,7 @@ def add_fanchart_traces(
     ]
 
 
-def hex_to_rgb(hex_string, opacity=1):
-    """Converts a hex color to rgb"""
-    hex_string = hex_string.lstrip("#")
-    hlen = len(hex_string)
-    rgb = [int(hex_string[i : i + hlen // 3], 16) for i in range(0, hlen, hlen // 3)]
-    rgb.append(opacity)
-    return f"rgba{tuple(rgb)}"
-
-
-def render_hovertemplate(vector: str, interval: Optional[str]):
+def render_hovertemplate(vector: str, interval: Optional[str]) -> str:
     if vector.startswith(("AVG_", "INTVL_")) and interval is not None:
         if interval == "daily":
             return "(%{x|%b} %{x|%-d}, %{x|%Y}, %{y})<br>"
@@ -198,7 +193,7 @@ def date_to_interval_conversion(
 
 
 # pylint: disable=too-many-branches
-def check_and_format_observations(obsfile: Path):
+def check_and_format_observations(obsfile: Path) -> dict:
     with open(obsfile, "r") as stream:
         try:
             obsfile_data = yaml.safe_load(stream)
@@ -225,7 +220,7 @@ def check_and_format_observations(obsfile: Path):
                 "The observation file lacks a `smry` section, which is mandatory for observations "
                 "to work with this plugin."
             ) from exc
-        except TypeError:
+        except TypeError as exc:
             raise TypeError(
                 "The observation file's othermost level must be a dictionary, while the input "
                 f"file's outermost level is of type {type(obslist)}."
