@@ -18,6 +18,7 @@ from webviz_config.webviz_store import webvizstore
 from webviz_config.common_cache import CACHE
 
 from webviz_subsurface._models import EnsembleSetModel
+from webviz_subsurface._models import caching_ensemble_set_model_factory
 from .._abbreviations.reservoir_simulation import (
     simulation_vector_description,
     simulation_unit_reformat,
@@ -87,10 +88,12 @@ class ReservoirSimulationTimeSeries(WebvizPluginABC):
 be plotted together with their non-historical counterparts as reference lines, and they are \
 therefore not selectable as vectors to plot initially.
 
-?> The `obsfile` is a common (optional) file for all ensembles, which currently has to be made \
-manually. [An example of the format can be found here]\
-(https://github.com/equinor/webviz-subsurface-testdata/blob/master/\
-reek_history_match/share/observations/observations.yml).
+?> The `obsfile` is a common (optional) file for all ensembles, which can be \
+converted from e.g. ERT and ResInsight formats using the [fmuobs]\
+(https://equinor.github.io/subscript/scripts/fmuobs.html) script. \
+[An example of the format can be found here]\
+(https://github.com/equinor/webviz-subsurface-testdata/blob/master/reek_history_match/share/\
+observations/observations.yml).
 
 !> It is **strongly recommended** to keep the data frequency to a regular frequency (like \
 `monthly` or `yearly`). This applies to both csv input and when reading from `UNSMRY` \
@@ -166,24 +169,23 @@ folder, to avoid risk of not extracting the right data.
                 sorted(pd.to_datetime(self.smry["DATE"]).unique())
             )
         elif ensembles:
-            self.emodel = EnsembleSetModel(
-                ensemble_paths={
-                    ens: webviz_settings.shared_settings["scratch_ensembles"][ens]
-                    for ens in ensembles
-                }
-            )
-            self.smry = self.emodel.load_smry(
-                time_index=self.time_index, column_keys=self.column_keys
-            )
-
-            self.smry_meta = self.emodel.load_smry_meta(
-                column_keys=self.column_keys,
+            self.emodel: EnsembleSetModel = (
+                caching_ensemble_set_model_factory.get_or_create_model(
+                    ensemble_paths={
+                        ens: webviz_settings.shared_settings["scratch_ensembles"][ens]
+                        for ens in ensembles
+                    },
+                    time_index=self.time_index,
+                    column_keys=self.column_keys,
+                )
             )
+            self.smry = self.emodel.get_or_load_smry_cached()
+            self.smry_meta = self.emodel.load_smry_meta()
         else:
             raise ValueError(
                 'Incorrent arguments. Either provide a "csvfile" or "ensembles"'
             )
-        if any([col.startswith(("AVG_", "INTVL_")) for col in self.smry.columns]):
+        if any(col.startswith(("AVG_", "INTVL_")) for col in self.smry.columns):
             raise ValueError(
                 "Your data set includes time series vectors which have names starting with"
                 "'AVG_' and/or 'INTVL_'. These prefixes are not allowed, as they are used"
@@ -797,6 +799,8 @@ folder, to avoid risk of not extracting the right data.
             cum_interval: str,
         ) -> Union[EncodedFile, str]:
             """Callback to download data based on selections"""
+            if data_requested is None:
+                raise PreventUpdate
 
             # Combine selected vectors
             vectors = [vector1]
@@ -866,18 +870,14 @@ folder, to avoid risk of not extracting the right data.
 
             # : is replaced with _ in filenames to stay within POSIX portable pathnames
             # (e.g. : is not valid in a Windows path)
-            return (
-                WebvizPluginABC.plugin_data_compress(
-                    [
-                        {
-                            "filename": f"{vector.replace(':', '_')}.csv",
-                            "content": df.get("stat", df["data"]).to_csv(index=False),
-                        }
-                        for vector, df in dfs.items()
-                    ]
-                )
-                if data_requested
-                else ""
+            return WebvizPluginABC.plugin_data_compress(
+                [
+                    {
+                        "filename": f"{vector.replace(':', '_')}.csv",
+                        "content": df.get("stat", df["data"]).to_csv(index=False),
+                    }
+                    for vector, df in dfs.items()
+                ]
             )
 
         @app.callback(
