@@ -23,6 +23,7 @@ from webviz_config import WebvizConfigTheme
 from webviz_config import WebvizSettings
 
 from webviz_subsurface._models import EnsembleSetModel
+from webviz_subsurface._models import caching_ensemble_set_model_factory
 from .._abbreviations.reservoir_simulation import (
     simulation_vector_base,
     simulation_vector_description,
@@ -137,59 +138,57 @@ folder, to avoid risk of not extracting the right data.
                 "'monthly' or 'yearly', as the statistics require the same dates throughout an"
                 "ensemble."
             )
-        self.emodel = EnsembleSetModel(
-            ensemble_paths={
-                ens: webviz_settings.shared_settings["scratch_ensembles"][ens]
-                for ens in ensembles
-            }
-        )
-        self.smry = self.emodel.load_smry(
-            time_index=self.time_index, column_keys=self.column_keys
+        self.emodel: EnsembleSetModel = (
+            caching_ensemble_set_model_factory.get_or_create_model(
+                ensemble_paths={
+                    ens: webviz_settings.shared_settings["scratch_ensembles"][ens]
+                    for ens in ensembles
+                },
+                time_index=self.time_index,
+                column_keys=self.column_keys,
+            )
         )
+        self.smry = self.emodel.get_or_load_smry_cached()
+        self.smry_meta = self.emodel.load_smry_meta()
 
-        self.smry_meta = self.emodel.load_smry_meta(
-            column_keys=self.column_keys,
-        )
         self.field_totals = [
             col for col in self.smry.columns if fnmatch.fnmatch(col, "F[OWG]PT")
         ]
+        self.rec_ensembles = set()
         if self.field_totals:
-            self.smry_init_prod = pd.concat(
-                [
-                    df[["ENSEMBLE", "DATE"] + self.field_totals][
-                        df["DATE"] == min(df["DATE"])
-                    ]
-                    for _, df in self.smry.groupby("ENSEMBLE")
-                ]
-            )
-        else:
-            total_smry = self.emodel.load_smry(
-                time_index=self.time_index,
-                column_keys=["F[OWG]PT"],
-            )
-            self.smry_init_prod = pd.concat(
+            smry_init_prod = pd.concat(
                 [
                     df[df["DATE"] == min(df["DATE"])]
-                    for _, df in total_smry.groupby("ENSEMBLE")
+                    for _, df in self.smry[
+                        ["ENSEMBLE", "DATE"] + self.field_totals
+                    ].groupby("ENSEMBLE")
                 ]
             )
-        self.rec_ensembles = set(self.smry["ENSEMBLE"].unique())
-        for col in self.smry_init_prod.columns:
-            if col not in ReservoirSimulationTimeSeriesRegional.ENSEMBLE_COLUMNS:
-                for ens in self.smry_init_prod["ENSEMBLE"].unique():
-                    if any(
-                        self.smry_init_prod[self.smry_init_prod["ENSEMBLE"] == ens][col]
-                        > 0.0001
-                    ):
-                        warnings.warn(
-                            f"Ensemble '{ens}' has initial production above 0, can therefore not"
-                            " calculate recovery for this ensemble (FOPT, FGPT and/or FWPT > 0)."
-                            " This ensemble probably includes restarts where we were not able to"
-                            " identify the filepaths to the original cases. Note that RESTART"
-                            " paths with more than 72 characters are not supported due to a"
-                            " simulator metadata file format limitation."
-                        )
-                        self.rec_ensembles.discard(ens)
+            for col in smry_init_prod.columns:
+                if col not in ReservoirSimulationTimeSeriesRegional.ENSEMBLE_COLUMNS:
+                    for ens in smry_init_prod["ENSEMBLE"].unique():
+                        if any(
+                            smry_init_prod[smry_init_prod["ENSEMBLE"] == ens][col]
+                            > 0.0001
+                        ):
+                            warnings.warn(
+                                f"Ensemble '{ens}' has initial production above 0, can"
+                                " therefore not calculate recovery for this ensemble"
+                                " (FOPT, FGPT and/or FWPT > 0)."
+                                " This ensemble probably includes restarts where we were"
+                                " not able to identify the filepaths to the original cases."
+                                " Note that RESTART paths with more than 72 characters are"
+                                " not supported due to a simulator metadata file format"
+                                " limitation."
+                            )
+                        else:
+                            self.rec_ensembles.add(ens)
+        else:
+            warnings.warn(
+                "No production vectors (FOPT, FGPT or FWPT) found for the provided"
+                " ensembles. Recoveries can not be calculated from inplace volumes."
+            )
+
         self.smry_cols: List[str] = []
         for col in self.smry.columns:
             if (
@@ -211,10 +210,8 @@ folder, to avoid risk of not extracting the right data.
         self.initial_vector = (
             initial_vector
             if any(
-                [
-                    col.startswith((f"{initial_vector}:", f"{initial_vector}_"))
-                    for col in self.smry_cols
-                ]
+                col.startswith((f"{initial_vector}:", f"{initial_vector}_"))
+                for col in self.smry_cols
             )
             else simulation_vector_base(self.smry_cols[0])
         )
@@ -1270,10 +1267,10 @@ def per_real_calculations(
 
 def calc_statistics(df: pd.DataFrame) -> pd.DataFrame:
     # Switched P10 and P90 due to convention in petroleum industry
-    def p10(x: np.array) -> float:
+    def p10(x: np.ndarray) -> float:
         return np.nanpercentile(x, q=90)
 
-    def p90(x: np.array) -> float:
+    def p90(x: np.ndarray) -> float:
         return np.nanpercentile(x, q=10)
 
     stat_dfs = []
